@@ -1,5 +1,5 @@
 moult <- function(formula, data = NULL, start = NULL, type = 2, method = "BFGS",
-                  fixed = NULL, fixed.val = NULL) 
+                  fixed = NULL, fixed.val = NULL, prec = 0.02) 
     
 { f.dat <- data.frame(data)
   
@@ -84,18 +84,18 @@ moult <- function(formula, data = NULL, start = NULL, type = 2, method = "BFGS",
         
             pdur <- p[1:p1]
             pstart <- p[(p1 + 1):(p1 + p2)]
-            psd <- p[(p1 + p2 + 1):no.params]
+            psd <- exp(p[(p1 + p2 + 1):no.params])
         
             P <- sum(log(1 - pnorm(M0, mean = mf0 %*% pstart, 
-                                   sd = sd0 %*% exp(psd) )))
+                                   sd = sd0 %*% psd )))
             
             Q <- sum (log(pnorm(MTime, mean = mfInd %*% pstart, 
-                                sd = sdInd %*% exp(psd)) - 
+                                sd = sdInd %*% psd) - 
                           pnorm(MTime - (dfInd %*% pdur), mean = mfInd %*% pstart, 
-                                sd = sdInd %*% exp(psd)) ))
+                                sd = sdInd %*% psd)))
    	
             R <- sum(log(pnorm(M1 - (df1 %*% pdur), mean = mf1 %*% pstart, 
-                               sd = sd1 %*% exp(psd))))
+                               sd = sd1 %*% psd)))
   
             loglik <- P + Q + R
             nobs <- sum(length(M0), length(M1), length(MInd))
@@ -119,24 +119,24 @@ moult <- function(formula, data = NULL, start = NULL, type = 2, method = "BFGS",
         
             pdur <- p[1:p1]
             pstart <- p[(p1 + 1):(p1 + p2)]
-            psd <- p[(p1 + p2 + 1):no.params]
+            psd <- exp(p[(p1 + p2 + 1):no.params])
         
             P.p <- log ( 1 - pnorm(M0, mean = mf0 %*% pstart,
-                                   sd = sd0 %*% exp(psd)))
+                                   sd = sd0 %*% psd))
             P <- sum(P.p[P.p > -Inf])
    
-            MIndmin <- pmax(0, MInd - 0.02)
-            MIndmax <- pmin(MInd + 0.02, 1)
+            MIndmin <- pmax(0, MInd - prec)
+            MIndmax <- pmin(MInd + prec, 1)
   
             q <- sum(log(pnorm(MTime - MIndmin * (dfInd %*% pdur),
                                mean = mfInd %*% pstart, 
-                               sd = sdInd %*% exp(psd)) -
+                               sd = sdInd %*% psd) -
                          pnorm(MTime - MIndmax * (dfInd %*% pdur),
                                mean = mfInd %*% pstart, 
-                               sd = sdInd %*% exp(psd))))
+                               sd = sdInd %*% psd)))
               
             R <- sum ( log ( pnorm (M1 - (df1 %*% pdur), mean = mf1 %*% pstart, 
-                                    sd = sd1 %*% exp(psd)) ))
+                                    sd = sd1 %*% psd) ))
 
             loglik <- P + q + R
             return(-loglik)
@@ -160,25 +160,30 @@ moult <- function(formula, data = NULL, start = NULL, type = 2, method = "BFGS",
         
           pdur <- params[1:p1]
           pstart <- params[(p1 + 1):(p1 + p2)]
-          psd <- params[(p1 + p2 + 1):no.params]  
+          psd <- exp(params[(p1 + p2 + 1):no.params]  )
         
-          MIndmin <- pmax(0, MInd - 0.02)
-          MIndmax <- pmin(MInd + 0.02, 1)
+          MIndmin <- pmax(0, MInd - prec)
+          MIndmax <- pmin(MInd + prec, 1)
        
           qq <- sum(log(pnorm(MTime - MIndmin * (dfInd %*% pdur),
                               mean = mfInd %*% pstart, 
-                              sd = sdInd %*% exp(psd)) -
+                              sd = sdInd %*% psd) -
                         pnorm(MTime - MIndmax * (dfInd %*% pdur),
                               mean = mfInd %*% pstart, 
-                              sd = sdInd %*% exp(psd))))
+                              sd = sdInd %*% psd)))
       
           Q <- sum(log(pnorm(MTime, mean = mfInd %*% pstart,
-                             sd = sdInd %*% exp(psd)) - 
+                             sd = sdInd %*% psd) - 
                        pnorm(MTime - (dfInd %*% pdur),
                              mean = mfInd %*% pstart,
-                             sd = sdInd %*% exp(psd))))
+                             sd = sdInd %*% psd)))
 
-          if (Q < qq) stop("Q < qq, type 3")
+          if (!is.na(Q) & !is.na(qq)) {
+            if (Q < qq) {
+              stop("Q < qq, type 3")
+              cat(list(Q = Q, qq = qq, params = params))
+            }
+          }
           nobs <- length(MInd)
           loglik <- qq - Q                      # eq.6
           return(-loglik)
@@ -203,27 +208,27 @@ moult <- function(formula, data = NULL, start = NULL, type = 2, method = "BFGS",
         
             pdur <- p[1:p1]
             pstart <- p[(p1 + 1):(p1 + p2)]
-            psd <- p[(p1 + p2 + 1):no.params]
+            psd <- exp(p[(p1 + p2 + 1):no.params])
  	
-            MIndmin <- pmax(0, MInd - 0.02)
-            MIndmax <- pmin(MInd + 0.02, 1)
+            MIndmin <- pmax(0, MInd - prec)
+            MIndmax <- pmin(MInd + prec, 1)
        
             qq <- sum(log(pnorm(MTime - MIndmin * (dfInd %*% pdur),
                                 mean = mfInd %*% pstart, 
-                                sd = sdInd %*% exp(psd)) -
+                                sd = sdInd %*% psd) -
                           pnorm(MTime - MIndmax * (dfInd %*% pdur),
                                 mean = mfInd %*% pstart, 
-                                sd = sdInd %*% exp(psd))))
+                                sd = sdInd %*% psd)))
   
             P1 <- sum ( log (pnorm(MTime, mean = mfInd %*% pstart,
-                                   sd = sdInd %*% exp(psd))))
+                                   sd = sdInd %*% psd)))
             
             R <- sum ( log ( pnorm (M1 - (df1 %*% pdur), 
                                     mean = mf1 %*% pstart, 
-                                    sd = sd1 %*% exp(psd)) ))
+                                    sd = sd1 %*% psd) ))
 
             P2 <- sum ( log (pnorm(M1, mean = mf1 %*% pstart, 
-                                   sd = sd1 %*% exp(psd))))
+                                   sd = sd1 %*% psd)))
 
             nobs <- sum(length(M1), length(MInd))
             loglik <- qq - P1 + R - P2
@@ -248,28 +253,28 @@ moult <- function(formula, data = NULL, start = NULL, type = 2, method = "BFGS",
             
             pdur <- p[1:p1]
             pstart <- p[(p1 + 1):(p1 + p2)]
-            psd <- p[(p1 + p2 + 1):no.params]
+            psd <- exp(p[(p1 + p2 + 1):no.params])
         
             P <- sum ( log ( 1 - pnorm(M0, mean = mf0 %*% pstart, 
-                                       sd = sd0 %*% exp(psd))))
+                                       sd = sd0 %*% psd)))
     
             R1 <- sum ( log ( 1 - pnorm (M0 - (df0 %*% pdur), 
                                          mean = mf0 %*% pstart, 
-                                         sd = sd0 %*% exp(psd)) ))
+                                         sd = sd0 %*% psd) ))
 
-            MIndmin <- pmax(0, MInd - 0.02)
-            MIndmax <- pmin(MInd + 0.02, 1)
+            MIndmin <- pmax(0, MInd - prec)
+            MIndmax <- pmin(MInd + prec, 1)
        
             qq <- sum(log(pnorm(MTime - MIndmin * (dfInd %*% pdur),
                                 mean = mfInd %*% pstart, 
-                                sd = sdInd %*% exp(psd)) -
+                                sd = sdInd %*% psd) -
                           pnorm(MTime - MIndmax * (dfInd %*% pdur),
                                 mean = mfInd %*% pstart, 
-                                sd = sdInd %*% exp(psd))))
+                                sd = sdInd %*% psd)))
   
             R2 <- sum ( log (1 - pnorm (MTime - (dfInd %*% pdur), 
                                         mean = mfInd %*% pstart, 
-                                        sd = sdInd %*% exp(psd)) ))
+                                        sd = sdInd %*% psd) ))
 
             nobs <- sum(length(M0), length(MInd))
             loglik <- P - R1 + qq - R2
